#########################################################################
# R code
# File: bp4vs.r
#
# Reference: 
# Chen, H., Chong, T.T.L., and Bai, J. (2012).
#    Theory and applications of TAR model with two threshold variables.
#    Econometric Reviews, 31(2), 142-170.
#    DOI: 10.1080/07474938.2011.607100.
#########################################################################

bp4vs3 <- function(x,z1,z2,threshold){

N    = 500;
p    = 0.95;
null = 0;

n4reg     = modelest(x,z1,z2,c(1,2,3,4),threshold);
n3reg23   = modelest(x,z1,z2,c(1,2,2,3),threshold);
lr23      = length(x)*(n3reg23$rss-n4reg$rss)/(n4reg$rss);
bpvalue23 = rep(0,N);

for(j in 1:N){
   sample23 = x;
   l        = length(x);
   cc1      = n3reg23$cc1;
   cc23     = n3reg23$cc23;
   cc4      = n3reg23$cc4;
  
   sample23[cc1]  = x[cc1]-n3reg23$re1+sample(n4reg$re1,replace=TRUE);
   sample23[cc23] = x[cc23]-n3reg23$re23+sample(c(n4reg$re2,n4reg$re3),replace=TRUE)[1:length(cc23)];
   sample23[cc4]  = x[cc4]-n3reg23$re4+sample(n4reg$re4,replace=TRUE);
          
   sn4reg   = modelest(sample23,z1,z2,c(1,2,3,4),threshold);
   sn3reg23 = modelest(sample23,z1,z2,c(1,2,2,3),threshold);
   bp23     = length(sample23)*(sn3reg23$rss-sn4reg$rss)/(sn4reg$rss);
   bpvalue23[j] = bp23;
}

print("the statistics value for c(1,2,3,4) to c(1,2,2,3)")
print(lr23); 
print("the 95% bootstrap critical value:");
print(sort(bpvalue23)[N*p]); 

if(lr23<sort(bpvalue23)[N*p]){
  print("it is c(1,2,2,3).  Go on to test 3 vs. 2");
  return(0);}
else{
  null    = 1;
  n3reg12 = modelest(x,z1,z2,c(1,1,2,3),threshold);
  lr12    = length(x)*(n3reg12$rss-n4reg$rss)/(n4reg$rss);
  bpvalue12 = rep(0,N);
  for(j in 1:N){
    sample12 = x;
    l        = length(x);
    cc12     = n3reg12$cc12;
    cc3      = n3reg12$cc3;
    cc4      = n3reg12$cc4;
    
    sample12[cc3]  = x[cc3]-n3reg12$re3+sample(n4reg$re3,replace=TRUE);
    sample12[cc12] = x[cc12]-n3reg12$re12+sample(c(n4reg$re1,n4reg$re2),replace=TRUE)[1:length(cc12)];
    sample12[cc4]  = x[cc4]-n3reg12$re4+sample(n4reg$re4,replace=TRUE);
      
    sn4reg   = modelest(sample12,z1,z2,c(1,2,3,4),threshold);
    sn3reg12 = modelest(sample12,z1,z2,c(1,1,2,3),threshold);
    bp12     = length(sample12)*(sn3reg12$rss-sn4reg$rss)/(sn4reg$rss);
    bpvalue12[j] = bp12;
  }
  
  print("the statistics vale for c(1,2,3,4) to c(1,1,2,3)")
  print(lr12); 
  print("the 95% bootstrap critical value:");
  print(sort(bpvalue12)[N*p]); 
  
  if(lr12<sort(bpvalue12)[N*p]){
    print("it is c(1,1,2,3).  Go on to test 3 vs. 2");
    return(0);}
  else{
    null    = 2;
    n3reg13 = modelest(x,z1,z2,c(1,2,1,3),threshold);
    lr13    = length(x)*(n3reg13$rss-n4reg$rss)/(n4reg$rss);
    bpvalue13 = rep(0,N);
    for(j in 1:N){
      sample13 = x;
      l        = length(x);
      cc13     = n3reg13$cc13;
      cc2      = n3reg13$cc2;
      cc4      = n3reg13$cc4;
      
      sample13[cc2]  = x[cc2]-n3reg13$re2+sample(n4reg$re2,replace=TRUE);
      sample13[cc13] = x[cc13]-n3reg13$re13+sample(c(n4reg$re1,n4reg$re3),replace=TRUE)[1:length(cc13)];
      sample13[cc4]  = x[cc4]-n3reg13$re4+sample(n4reg$re4,replace=TRUE);
        
      sn4reg   = modelest(sample13,z1,z2,c(1,2,3,4),threshold);
      sn3reg13 = modelest(sample13,z1,z2,c(1,2,1,3),threshold);
      bp13     = length(sample13)*(sn3reg13$rss-sn4reg$rss)/(sn4reg$rss);
      bpvalue13[j] = bp13;
    }
    
    print("the statistics value for c(1,2,3,4) to c(1,2,1,3)")
    print(lr13); 
    print("the 95% bootstrap critical value:");
    print(sort(bpvalue13)[N*p]);
    
    if(lr13<sort(bpvalue13)[N*p]){
      print("it is c(1,2,1,3).  Go on to test 3 vs 2");
      return(0);}
    else{
      null    = 3;
      n3reg14 = modelest(x,z1,z2,c(1,2,3,1),threshold);
      lr14    = length(x)*(n3reg14$rss-n4reg$rss)/(n4reg$rss);
      bpvalue14 = rep(0,N);
      for(j in 1:N){
        sample14 = x;
        l        = length(x);
        cc14     = n3reg14$cc14;
        cc2      = n3reg14$cc2;
        cc3      = n3reg14$cc3;
    
        sample14[cc2]  = x[cc2]-n3reg14$re2+sample(n4reg$re2,replace=TRUE);
        sample14[cc14] = x[cc14]-n3reg14$re14+sample(c(n4reg$re1,n4reg$re4),replace=TRUE)[1:length(cc14)];
        sample14[cc3]  = x[cc3]-n3reg14$re3+sample(n4reg$re3,replace=TRUE);
         
        sn4reg   = modelest(sample14,z1,z2,c(1,2,3,4),threshold);
        sn3reg14 = modelest(sample14,z1,z2,c(1,2,3,1),threshold);
        bp14     = length(sample14)*(sn3reg14$rss-sn4reg$rss)/(sn4reg$rss);
        bpvalue14[j] = bp14;
      }
      
      print("the statistics value for c(1,2,3,4) to c(1,2,3,1)")
      print(lr14); 
      print("the 95% bootstrap critical value:");
      print(sort(bpvalue14)[N*p]); 
      
      if(lr14<sort(bpvalue14)[N*p]){
        print("it is c(1,2,3,1).  Go on to test 3 vs. 2");
        return(0);}
      else{
        null    = 4;
        n3reg24 = modelest(x,z1,z2,c(1,2,3,2),threshold);
        lr24    = length(x)*(n3reg24$rss-n4reg$rss)/(n4reg$rss);
        bpvalue24 = rep(0,N);
        for(j in 1:N){
          sample24 = x;
          l        = length(x);
          cc24     = n3reg24$cc24;
          cc1      = n3reg24$cc1;
          cc3      = n3reg14$cc3;
     
          sample24[cc1]  = x[cc1]-n3reg24$re1+sample(n4reg$re1,replace=TRUE);
          sample24[cc24] = x[cc24]-n3reg24$re24+sample(c(n4reg$re2,n4reg$re4),replace=TRUE)[1:length(cc24)];
          sample24[cc3]  = x[cc3]-n3reg24$re3+sample(n4reg$re3,replace=TRUE);
          
          sn4reg   = modelest(sample24,z1,z2,c(1,2,3,4),threshold);
          sn3reg24 = modelest(sample24,z1,z2,c(1,2,3,2),threshold);
          bp24     = length(sample24)*(sn3reg24$rss-sn4reg$rss)/(sn4reg$rss);
          bpvalue24[j] = bp24;
        }
        
        print("the statistics value for c(1,2,3,4) to c(1,2,3,2)")
        print(lr24); 
        print("the 95% boostrap critical value:");
        print(sort(bpvalue24)[N*p]); 
        
        if(lr24<sort(bpvalue24)[N*p]){
          print("it is c(1,2,3,2).  Go on to test 3 vs. 2");
          return(0);}
        else {
          null    = 5;
          n3reg34 = modelest(x,z1,z2,c(1,2,3,3),threshold);
          lr34    = length(x)*(n3reg34$rss-n4reg$rss)/(n4reg$rss);
          bpvalue34 = rep(0,N);
          for(j in 1:N){
            sample34 = x;
            l        = length(x);
            cc34 = n3reg34$cc34;
            cc1  = n3reg34$cc1;
            cc2  = n3reg34$cc2;
            sample34[cc1]  = x[cc1]-n3reg34$re1+sample(n4reg$re1,replace=TRUE);
            sample34[cc34] = x[cc34]-n3reg34$re34+sample(c(n4reg$re3,n4reg$re4),replace=TRUE)[1:length(cc34)];
            sample34[cc2]  = x[cc2]-n3reg34$re2+sample(n4reg$re2,replace=TRUE);
      
            sn4reg   = modelest(sample34,z1,z2,c(1,2,3,4),threshold);
            sn3reg34 = modelest(sample34,z1,z2,c(1,2,3,3),threshold);
            bp34     = length(sample34)*(sn3reg34$rss-sn4reg$rss)/(sn4reg$rss);
            bpvalue34[j] = bp34;
          }
           
          print("the statistics value for c(1,2,3,4) to c(1,2,3,3)")
          print(lr34); 
          print("the 95% bootstrap critical value:");
          print(sort(bpvalue34)[N*p]);
          
          if(lr34>=sort(bpvalue34)[N*p]){
            null=6;
            print("it is a four-regime model");
            return(0);                               
         else {
            print("go on to test c(1,2,3,3) to 2 regimes model");
            null=0;
           n3reg34=modelest(x,z1,z2,c(1,2,3,3),threshold);
         }
   } #  end loop if(lr24<sort(bpvalue24)[N*p])
  }  #  end loop if(lr14<sort(bpvalue14)[N*p])
 }   #  end loop if(lr13<sort(bpvalue13)[N*p])
  }  #  end loop if(lr12<sort(bpvalue12)[N*p])
}    #  end loop if(lr23<sort(bpvalue23)[N*p])
}    #  end function